package simpleorm.dataset;

import java.sql.ResultSet;

import simpleorm.utils.SException;

/** Represents Double field meta data. */

public class SFieldDouble extends SFieldScalar {
	static final long serialVersionUID = 3L;

	public SFieldDouble(SRecordMeta meta, String columnName,
			SFieldFlags... pvals) {
		super(meta, columnName, pvals);
	}

	/**
	 * Specidalizes abstract method to actually query a column from a result
	 * set.
	 */
	public Object queryFieldValue(ResultSet rs, int sqlIndex) throws Exception {
		double res = rs.getDouble(sqlIndex);
		if (rs.wasNull()) // ie. last operation!
			return null;
		else
			return new Double(res);
	}

	protected Object convertToDataSetFieldType(Object raw) throws Exception {
		if (raw instanceof Double)
			return raw;
		if (raw == null)
			return null; // Follows JDBC
		if (raw instanceof Number)
			return new Double(((Number) raw).intValue());
		if (raw instanceof String) {
			return Double.parseDouble((String) raw);
		}
		throw new SException.Data("Cannot convert " + raw + " to Double.");
	}

	@Override
	public String defaultSqlDataType() {
		return "DOUBLE PRECISION";
	}
	
	public boolean isCompatible(SFieldScalar field) {

		if (!(field instanceof SFieldDouble))
			return false;
		return true;
	}

	@Override
	public
	int javaSqlType() {
		return java.sql.Types.DOUBLE;
	}

}
