header {
package org.hmn.jrsd;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.HashSet;
}

class JrsdParser extends Parser;
options {
  k = 10;
  //buildAST = true;
}
{
  ArrayList lits = new ArrayList();
  ArrayList errs = new ArrayList();
  void lc() { 
    JrsdLexer lx = (JrsdLexer) getInputState().getInput().getInput();
    String lcs= lx.popLiteralChars();
    //System.out.println(lcs);
    lits.add(lcs);
  };
  List literals() { ArrayList lst = lits; lits = new ArrayList(); return lst; }
  public void reportError(RecognitionException ex) {
    errs.add(ex);
    super.reportError(ex);
  }

}

jrsdspec 
  returns [JrsdSpec spec = new JrsdSpec()]
  :  
  preamble[spec]
  imports[spec]
  globalsets[spec]
  tabledefs[spec]    
  EOF 
  ;

preamble[JrsdSpec spec]
  :
"jrsd" INTNR ("encoding" ID)?  (SC)?
(
  "database" a:STRING (b:STRING)? SC
  { 
    spec.setDatabaseType(a.getText()); 
    spec.setDatabaseSubtype( b == null ? "" : b.getText() ); 
  }
)?
  ;

imports[JrsdSpec spec]
  :
  (
    "import" i:STRING (SC)?
    { spec.addImport(i.getText()); }
  )*
  ;
  
globalsets[JrsdSpec spec]
  :
  (
    setdecl[spec]
  | fieldprefix[spec]
  )* 
  ;

setdecl[JrsdSpec spec]
  :
"columnlist" qn:STRING "as" a:ID SC
{ spec.addGlobalColumnList(a.getText(), qn.getText()); }
  ;
      
fieldprefix[JrsdSpec spec]
  :
"field" "prefix" fp:STRING SC
{ spec.setFieldPrefix(fp.getText()); }
  ;      
  
tabledefs[JrsdSpec spec]    
  :  
    ( tabledef[spec] )*  
  ;
    
tabledef[JrsdSpec spec]    
  { List c; }
  :
  {
    String a = "";
    TableDef tdef = new TableDef();
    spec.addTableDef(tdef);
  }
    (c1:INCLUDEDCOMMENT)?
    "table" a=torcname "version" b:INTNR (lit:LT)? c=directives (COL mappingdecls[tdef])?
      { 
        tdef.setComment(c1);
        tdef.setName(a);
        tdef.setVersion(b);
        tdef.setLiteralSql(lit);
        tdef.addDirectives(c);
      }
    LPAR 
      ( 
        coldef[tdef]         
      )* 
      (
        extraSql:LT
        {tdef.setExtraSql(extraSql);}
      )?
    RPAR
    (
      postSql:LT
      {tdef.setPostSql(postSql);}
    )?
    SC
  ;

torcname
  returns [String name = ""]
  :
  (
    a:ID
    { name = a.getText(); }
  | b:STRING
    { name = b.getText(); }
  )
  ;
  
mappingdecls[TableDef td]  
  { MappingDecl md = null; }
  :
    (c1:INCLUDEDCOMMENT)? md=mappingdecl
  { md.setComment(c1); td.addMappingDecl(md); }    
      (COM (c2:INCLUDEDCOMMENT)? md=mappingdecl
  { md.setComment(c2); td.addMappingDecl(md); }      
      )*
  ;
  
mappingdecl
  returns [MappingDecl md = null]
  { List flags = null; }
  :    
    "simpleorm" cn:STRING ("extends" (ab:"abstract")? xn:STRING)? "as" i:ID 
       flags=directives (lc:LT)?
    { 
      md = new MappingDecl("simpleorm", cn.getText(), i.getText()); 
      md.setSuperClass(xn == null ? null : xn.getText());
      md.setSuperClassAbstract(ab != null);
      md.setExtraCode(lc);
      md.setSormFlags(flags);
    }
  | "transport" tn:STRING "as" j:ID
    { md = new MappingDecl("transport", tn.getText(), j.getText()); }
  | "fieldset" fsn:STRING "as" k:ID
    { md = new MappingDecl("fieldset", fsn.getText(), k.getText()); }
  ;
    
coldef[TableDef td]  
  { ColumnTypeDef ct = null; ArrayList fds = null; String a = ""; List ds;}
  :
    {ArrayList pks = null; ColumnDef cd = null;}
    (ic:INCLUDEDCOMMENT)?    
    (
      "primary" "key" LPAR pks=idlist RPAR (la:LT)? SC
        {cd = new ColumnDef(td, ic, pks, la); td.addColumnDef(cd);}    
    | ( 
        a=torcname  ct=sqltype  (lb:LT)? ds=directives
          {
          cd = new ColumnDef(td, ic, a, ct, lb); td.addColumnDef(cd);
          cd.addDirectives(ds);
          }
        (
          COL fielddecls[cd]
        )? 
        SC 
      )
    )  
  ;
  
fielddecls[ColumnDef cd]
  { FieldDecl a = null; }
  :    
    a=fielddecl[cd]
      {cd.add(a);} 
    (
      COM a=fielddecl[cd] 
        {cd.add(a);} 
    )*
  ;
  
fielddecl[ColumnDef cd]
  returns [FieldDecl fd = new FieldDecl(cd)]
  { FieldDecl sf = null; List dl = null; }  
  :
    (
      a:ID  {fd.setAlias(a);}
      (
      DOT b:ID {fd.setFieldName(b);}
    | DOT b2:STRING {fd.setFieldName(b2);}
      )?
      (
        LPAR
        sf=subfield[cd] {fd.addSubfield(sf);} 
        ( 
          COM sf=subfield[cd] {fd.addSubfield(sf);} 
        )*
        RPAR
      )?
      (
        (
           //here the keywords are allowed as types, so besides ID, we must
           //also look for them and enforce correct case                      
           "int" {fd.setExplicitSFieldTypeName("int");}         
         | "integer" {fd.setExplicitSFieldTypeName("Integer");}         
         | "long" {fd.setExplicitSFieldTypeName("Long");}         
         | "primitive_long" {fd.setExplicitSFieldTypeName("long");}         
         | "double" {fd.setExplicitSFieldTypeName("Double");}
         | "primitive_double" {fd.setExplicitSFieldTypeName("double");}
         | "boolean" {fd.setExplicitSFieldTypeName("Boolean");}
         | "primitive_boolean" {fd.setExplicitSFieldTypeName("boolean");}
         | "date"{fd.setExplicitSFieldTypeName("Date"); }
         | "bigdecimal" {fd.setExplicitSFieldTypeName("BigDecimal");}
         | "blob" {fd.setExplicitSFieldTypeName("Blob");}
           //ref is treated specially
         | "ref" y:ID { fd.setExplicitSFieldTypeName("REF#"+y.getText()); }   
         | c:ID {fd.setExplicitSFieldTypeName(c);}        
        )  
        (
          //here the keywords are 
          DBLCOL
          (  
           "int" { fd.setSormBaseType("int"); }         
         | "integer" { fd.setSormBaseType("Integer"); }         
         | "primitive_long" { fd.setSormBaseType("long"); }         
         | "long" { fd.setSormBaseType("Long"); }         
         | "primitive_double" { fd.setSormBaseType("double"); }
         | "double" { fd.setSormBaseType("Double"); }
         | "primitive_boolean" { fd.setSormBaseType("boolean"); }
         | "boolean" { fd.setSormBaseType("Boolean"); }
         | "date"{ fd.setSormBaseType("Date"); }
         | "bigdecimal" { fd.setSormBaseType("BigDecimal"); }
         | "bytes" { fd.setSormBaseType("byte[]"); }
         | h:ID { fd.setSormBaseType(h.getText()); }
          )
        )?
      )?
      dl=directives { fd.addDirectives(dl); }
    )
  ;

subfield[ColumnDef cd]
  returns [FieldDecl fd = new FieldDecl(cd)]
  :
    a:ID {fd.setAlias(a);}
    (
      DOT 
      (
      b:ID {fd.setFieldName(b);}
    | c:INTNR {fd.setFieldName(c);}
    | c2:STRING {fd.setFieldName(c2);}
      )
    )?
  ;
  
directives
  returns [List dl = new ArrayList()]
  :
    (
      d:DIRECTIVE { dl.add(d.getText()); }      
    )*
  ;
  
sqltype
  returns [ColumnTypeDef td = null]
  :
    "char" LPAR ac:INTNR RPAR
    { td = new ColumnTypeDef("char", ac.getText(), null); }
  | "varchar" LPAR a:INTNR RPAR
    { td = new ColumnTypeDef("varchar", a.getText(), null); }
  | "bigdecimal"
    { td = new ColumnTypeDef("bigdecimal", null, null); }
  | "integer"
    { td = new ColumnTypeDef("integer", null, null); }
  | "int"
    { td = new ColumnTypeDef("int", null, null); }
  | "primitive_long"
    { td = new ColumnTypeDef("primitive_long", null, null); }
  | "long"
    { td = new ColumnTypeDef("long", null, null); }
  | "primitive_double"
    { td = new ColumnTypeDef("primitive_double", null, null); }
  | "double"
    { td = new ColumnTypeDef("double", null, null); }
  | "date"
    { td = new ColumnTypeDef("date", null, null); }
  | "timestamp"
    { td = new ColumnTypeDef("timestamp", null, null); }
  | "time"
    { td = new ColumnTypeDef("time", null, null); }
  | "blob" LPAR aa:INTNR RPAR
    { td = new ColumnTypeDef("blob", aa.getText(), null); }
  | "clob" LPAR ab:INTNR RPAR
    { td = new ColumnTypeDef("clob", ab.getText(), null); }
  | "blob"
    { td = new ColumnTypeDef("blob", ""+Integer.MAX_VALUE, null); }
  | "clob"
    { td = new ColumnTypeDef("clob", ""+Integer.MAX_VALUE, null); }
  | "primitive_booleanbit"
    { td = new ColumnTypeDef("primitive_booleanbit", null, null); }
  | "booleanbit"
    { td = new ColumnTypeDef("booleanbit", null, null); }
  | "bytes"
    (
      LPAR bb:INTNR RPAR
    )?
    {      
      td = new ColumnTypeDef("bytes", (bb == null ? null : bb.getText()), null); 
    }    
  | "number"
    (
      LPAR b:INTNR (COM c:INTNR)? RPAR
    )?
    {
      String ctxt = c == null ? null : c.getText();
      td = new ColumnTypeDef("number", b.getText(), ctxt); 
    }    
  | //note that the string in "other" can not be one of the keywords (like "date", "blob", etc.
    "other" LPAR d:STRING RPAR
    { td = new ColumnTypeDef("other", d.getText(), null); }
  | "primitive_booleanchar" LPAR e:STRING COM f:STRING RPAR
    { td = new ColumnTypeDef("primitive_booleanchar", e.getText(), f.getText()); }
  | "booleanchar" LPAR g:STRING COM h:STRING RPAR
    { td = new ColumnTypeDef("booleanchar", g.getText(), h.getText()); }
  ;

idlist
  returns [ArrayList ids = null]
  :
  { String a = ""; String b = ""; }
    a=torcname { ids = new ArrayList(); ids.add(a);} 
    (
      COM b=torcname { ids.add(b); } 
    )*
  ;
  
class JrsdLexer extends Lexer;
options {
  k=2;
  charVocabulary = '\u0000'..'\uFFFE';
  caseSensitive=false;  
  caseSensitiveLiterals=false;
  testLiterals = false; 
}
{
  int crline = -1;
  StringBuffer literalChars = new StringBuffer();
  public String popLiteralChars() { 
    String s = literalChars.toString(); literalChars = new StringBuffer();
    return s;
  }
  /**
   * @see antlr.CharScanner#reportError(antlr.RecognitionException)
   */
  public void reportError(RecognitionException ex) {
    super.reportError(ex);
    throw new RuntimeException(ex);
  }  
  /**
   * @see antlr.CharScanner#reportError(java.lang.String)
   */
  public void reportError(String s) {
    throw new RuntimeException(s);
  } 
}

LPAR : '(' ;
RPAR : ')' ;
SC : ';' ;
COL : ':' ;
DBLCOL : "::" ;
DOT : '.' ;
COM : ',' ;

WS : (' ' | '\t') { _ttype = Token.SKIP; } ;
CR : '\r'  
  { 
    _ttype = Token.SKIP; 
    /*System.out.println("CR"+inputState.getLine());*/ 
    newline(); 
    crline = inputState.getLine(); 
  } 
  ; 
LF : '\n'  
  { 
    _ttype = Token.SKIP; 
    /*System.out.println("NL"+inputState.getLine());*/ 
    if (inputState.getLine() > crline) newline(); 
  } 
  ;

ID
options { testLiterals = true; }
  : ('a'..'z' | '_') ('a'..'z' | '_' | '0'..'9')* ;
  
DIRECTIVE
options { testLiterals = false; }
  : '#'! ('a'..'z' | '_')* ;

INTNR : ('0'..'9')+ ;

LT : '{'!
       (
         ('\\'! '}' ) {literalChars.append('}');}  
       |
         ('\\'! '\\')
       | 
         {
           char cc = LA(1);
           if (cc == '\r') { newline(); crline = inputState.getLine(); }
           if (cc == '\n') { if (inputState.getLine() > crline) newline(); }
         }                      
         ~('\\' | '}') 
       )+ 
     '}'!
  {
    literalChars.append(getText());    
  }
  ;

INCLUDEDCOMMENT : 
  "/*"!
    ( options { greedy=false; } 
  : 
    {
      char cc = LA(1);
      if (cc == '\r') { newline(); crline = inputState.getLine(); }
      if (cc == '\n') { if (inputState.getLine() > crline) newline(); }
    }
      .
    )* 
  "*/"!
  ;
IGNOREDCOMMENT: "//" 
  (
    ~('\r'|'\n')
  )* 
  ( 
    {newline(); crline = inputState.getLine();}
    '\r'  
  )? 
  ( 
    {if (inputState.getLine() > crline) newline();}
    '\n' 
  )? 
  { _ttype = Token.SKIP; };


STRING 
options { testLiterals = false; }
 : '"'! ( ESCAPE | ~('"'|'\\') )* '"'! ;

/* " */
protected
ESCAPE
    :    '\\'
         ( 'n' { $setText("\n"); }
         | 'r' { $setText("\r"); }
         | 't' { $setText("\t"); }
         | '\'' { $setText("\'"); }
         | '"' { $setText("\""); }
         )
    ;
    
/*
 * qualifiedName abandonned as it forbidded use of keywords (for example you
 * could not use "jrsd" name as it was mistaken for a keyword)
 */
 