package org.hmn.jrsd.test;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.Arrays;

import oracle.jdbc.driver.OracleConnection;
import oracle.sql.BLOB;

import org.hmn.jrsd.rt.JrsdRtUtils;
import org.hmn.jrsd.rt.SormUtils;

import simpleorm.sessionjdbc.SSessionJdbc;

import junit.framework.TestCase;
/*
 * 
 * Cree par Harry Karadimas a 20 sept. 2006
 */
/**
 * 
 * @author karadimas
 */
public class BlobTests
extends OraJrsdTestCase
{
  static final String TABLE_DEF_PATH = "sql/table-TEST_BLOB_v1_1276001705.sql";
  public static final String tableName = 
    TEST_BLOB_gen.meta.getTableName();
  static final private byte[] testContents = {0,1,2,3,4,5};
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp() throws Exception {
    super.setUp();
    beginConnection("BlobTests setUp");
    if (!SormUtils.tableExists(tableName)) {
      System.out.println("creation de la table "+tableName);
      ddls(JrsdRtUtils.loadFile(TABLE_DEF_PATH));
    }   
    // empty_blob() is the function that initializes a blob in Oracle
    ddl("DELETE FROM TEST_BLOB");
    ddl("INSERT INTO TEST_BLOB(ID, NAME, SOMEBLOB)" +
        " VALUES (1, 'My first BLOB', empty_blob())");
    ddl("INSERT INTO TEST_BLOB(ID, NAME, SOMEBLOB)" +
        " VALUES (2, 'My null BLOB', null)");
    ddl("INSERT INTO TEST_BLOB(ID, NAME, SOMEBLOB)" +
        " VALUES (3, 'My third BLOB', empty_blob())");
    String sql = "SELECT SOMEBLOB FROM TEST_BLOB FOR UPDATE";
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    Connection cxn = session.getJdbcConnection();
    Statement st = cxn.createStatement();
    ResultSet rs = st.executeQuery(sql);
    BLOB blb = null;
    if (rs.next()) blb = (BLOB) rs.getBlob(1);
    OutputStream os = blb.getBinaryOutputStream();
    os.write(testContents);
    os.close();
    st.close();
    endConnection();
  }
  
  public void testBlobRead() throws Exception
  {
    SSessionJdbc session = beginConnection("testBlobRead");
    TEST_BLOB_gen rec = session.find(TEST_BLOB_gen.meta, 1);
    BLOB blb = (BLOB) rec.getObject(TEST_BLOB_gen.SOMEBLOB);
    InputStream in = blb.getBinaryStream();
    byte[] buf = readStreamBytes(in);
    //S ystem.out.println("Read buf of "+buf.length+" bytes");
    for (int i = 0; i < buf.length; i++) {
      //if (i > 0) S ystem.out.print(',');
      //S ystem.out.print(i);
    }
    assertTrue(Arrays.equals(buf, testContents));    
    in.close();
    assertEquals(1, buf[1]);
    endConnection();
  }
  
  private byte[] readStreamBytes(InputStream in) throws IOException {
    ByteArrayOutputStream bos = new ByteArrayOutputStream();
    int b; while ((b = in.read()) >= 0) bos.write(b);
    byte[] buf = bos.toByteArray();
    return buf;
  }

  public void testBlobNull() throws Exception
  {
    SSessionJdbc session = beginConnection("testBlobNull");
    TEST_BLOB_gen rec = session.find(TEST_BLOB_gen.meta, 2);    
    BLOB blb = (BLOB) rec.getObject(TEST_BLOB_gen.SOMEBLOB);
    //S ystem.out.println("blb="+blb);
    assertNull(blb);
    endConnection();
  }
    
}
