package org.hmn.jrsd.test;

import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import oracle.sql.BLOB;

import org.hmn.jrsd.rt.JrsdRtUtils;
import org.hmn.jrsd.rt.SormUtils;

import simpleorm.dataset.SFieldBlob;
import simpleorm.dataset.SFieldFlags;
import simpleorm.dataset.SFieldInteger;
import simpleorm.dataset.SFieldString;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;
import simpleorm.sessionjdbc.SSessionJdbc;

public class BlobTests2
extends OraJrsdTestCase
{
  private static final int MB = 1024 * 1024;
  private static final int GB = 1024 * MB;
  
  private static final String longString =
    "A super very long string but not that long in fact";
  
  public static final String tableName = 
    TEST_BLOB2.meta.getTableName();

  static final private byte[] testContents = {0,1,2,3,4,5};
  
  protected void setUp() throws Exception {
    super.setUp();
    SSessionJdbc s = beginConnection("BlobTests setUp");
    if (!SormUtils.tableExists(tableName)) {
      System.out.println("creation de la table "+tableName);
      //TEST_BLOB2.meta.
      //s.getDriver().dropTableNoError(TEST_BLOB2.meta.getTableName());
      ddl(s.getDriver().createTableSQL(TEST_BLOB2.meta));
    }   
    // empty_blob() is the function that initializes a blob in Oracle
    ddl("DELETE FROM TEST_BLOB");
    ddl("INSERT INTO TEST_BLOB(ID, NAME, SOMEBLOB)" +
        " VALUES (2, 'My null BLOB', null)");
    ddl("INSERT INTO TEST_BLOB(ID, NAME, SOMEBLOB)" +
        " VALUES (3, 'My third BLOB', empty_blob())");
    endConnection();
  }
  
  public void testRegularBlobCreation()
  throws Exception
  {
    SSessionJdbc s = beginConnection("testRegularBlobCreation");
    Connection cxn = s.getJdbcConnection();
    String insSql = "INSERT INTO TEST_BLOB(ID, NAME, SOMEBLOB)" +
        " VALUES (4, 'My null BLOB #4', ?)";
    PreparedStatement ps = cxn.prepareStatement(insSql);
    String str = longString;    
    byte[] bytes = str.getBytes();
    ByteArrayInputStream bis = new ByteArrayInputStream(bytes);
    try {
      ps.setBinaryStream(1, bis, bytes.length); // error in Oracle      
    }
    catch (Throwable t) {
      t.printStackTrace(System.err);
      ps.setBytes(1, str.getBytes("US-ASCII")); //more usual way to get content
    }
    ps.executeUpdate();
    endConnection();
    s.close();
  }
  
  public void testSimpleOrmBlobSupport1()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testSimpleOrmBlobSupport1");
    s.begin();
    TEST_BLOB2 rec = s.create(TEST_BLOB2.meta, 5);
    rec.setString(TEST_BLOB2.NAME, "The test number 5");
    rec.setBytes(TEST_BLOB2.SOMEBLOB, longString.getBytes("US-ASCII"));
    s.commit();
    s.close();
  }
  
  public void testSimpleOrmBlobSupport2()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testSimpleOrmBlobSupport1");
    s.begin();
    TEST_BLOB2 rec = s.create(TEST_BLOB2.meta, 6);
    rec.setString(TEST_BLOB2.NAME, "The test number 6");
    rec.setBytes(TEST_BLOB2.SOMEBLOB, testContents);
    s.commit();
    //now re-read
    s.begin();
    rec = s.find(TEST_BLOB2.meta, 6);
    byte[] readContents = rec.getBytes(TEST_BLOB2.SOMEBLOB);
    assertNotNull(readContents);
    assertEquals((byte)5, readContents[5]);
    s.commit();
    s.close();
  }
  
  public static class TEST_BLOB2 extends SRecordInstance {
    static SRecordMeta<TEST_BLOB2> meta = 
      new SRecordMeta<TEST_BLOB2>(TEST_BLOB2.class, "TEST_BLOB");
      
    static SFieldInteger ID = new SFieldInteger(meta, "ID", SFieldFlags.SPRIMARY_KEY);
    static SFieldString NAME = new SFieldString(meta, "NAME", 32);
    static SFieldBlob SOMEBLOB = new SFieldBlob(meta, "SOMEBLOB", 4 * GB);
    
    @Override
    public SRecordMeta<?> getMeta() {
      return meta;
    }
    
  }
  
}
