package org.hmn.jrsd.test;

import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import oracle.sql.BLOB;

import org.hmn.jrsd.rt.ClobCollectVisitor;
import org.hmn.jrsd.rt.JrsdRtUtils;
import org.hmn.jrsd.rt.ResultSetColVisitable;
import org.hmn.jrsd.rt.SormUtils;
import org.hmn.jrsd.test.ClobTests2.TEST_CLOB2;

import simpleorm.dataset.SFieldBlob;
import simpleorm.dataset.SFieldClob2;
import simpleorm.dataset.SFieldFlags;
import simpleorm.dataset.SFieldInteger;
import simpleorm.dataset.SFieldString;
import simpleorm.dataset.SQueryMode;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;
import simpleorm.sessionjdbc.SSessionJdbc;

public class ClobTests3
extends OraJrsdTestCase
{
  private static final int MB = 1024 * 1024;
  private static final int GB = 1024 * MB;
  
  private static final String longString =
    "A super very long string but not that long in fact";
  
  public static final String tableName = 
    TEST_CLOB3.meta.getTableName();

  static final private byte[] testContents = {0,1,2,3,4,5};
  
  protected void setUp() throws Exception {
    super.setUp();
    SSessionJdbc s = beginConnection("ClobTests setUp");
    if (!SormUtils.tableExists(tableName)) {
      System.out.println("creation de la table "+tableName);
      //TEST_CLOB3.meta.
      //s.getDriver().dropTableNoError(TEST_CLOB3.meta.getTableName());
      ddl(s.getDriver().createTableSQL(TEST_CLOB3.meta));
    }   
    // empty_blob() is the function that initializes a blob in Oracle
    ddl("DELETE FROM TEST_CLOB3");
    ddl("INSERT INTO TEST_CLOB3(ID, NAME, SOMECLOB)" +
        " VALUES (2, 'My null CLOB', null)");
    ddl("INSERT INTO TEST_CLOB3(ID, NAME, SOMECLOB)" +
        " VALUES (3, 'My third CLOB', empty_clob())");
    endConnection();
  }
   
  /**
   * Test the {@link SormUtils#updateExtra(SSessionJdbc, SRecordInstance, String)}
   * and {@link SormUtils#selectExtra(SSessionJdbc, SRecordInstance, String, ResultSetColVisitable)}
   * methods.
   * @throws Exception
   */
  public void testUpdateSelectExtra()
  throws Exception
  {
    String allowedChars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-@_()[]=+%!:/;.,?*";
    SSessionJdbc s = SSessionJdbc.open(ds, "testSelectExtra");
    s.begin();
    TEST_CLOB3 rec = s.create(TEST_CLOB3.meta, 5);
    rec.setString(TEST_CLOB3.NAME, "The test number 5");    
    s.commit();
    s.begin();
    rec = s.find(TEST_CLOB3.meta, 5);
    //make a 256K string of chars
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < 256 * 1024; i++) {
      sb.append(allowedChars.charAt((int)(Math.random() * allowedChars.length())));
    }
    String str256k = sb.toString();
    SormUtils.updateExtra(s, rec, "SOMECLOB = empty_clob()");
    ClobCollectVisitor visitor = new ClobCollectVisitor();
    SormUtils.selectExtra(s, rec, "SOMECLOB", visitor);
    Clob clb = visitor.getClob(); 
    try {
      // The following method fails in Oracle, because the string is too big
      clb.setString(0, str256k); //Oracle error : Invalid arguments      
    }
    catch (Exception ex) {
      ex.printStackTrace(System.err);
    }
    //less convenient method, but works
    SormUtils.writeToClob(str256k, clb);
    s.commit();
    s.close();    
  }
  
  public void testRegularClobUsage()
  throws Exception
  {
    SSessionJdbc s = SSessionJdbc.open(ds, "testRegularClobUsage");
    
    //make a 256K string of chars
    String allowedChars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-@_()[]=+%!:/;.,?*";
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < 256 * 1024; i++) {
      sb.append(allowedChars.charAt((int)(Math.random() * allowedChars.length())));
    }
    String str256k = sb.toString();    
    
    s.begin();
    TEST_CLOB3 rec = s.create(TEST_CLOB3.meta, 6);
    rec.setString(TEST_CLOB3.NAME, "The test number 6");
    s.flush();
    SormUtils.updateExtra(s, rec, "SOMECLOB = empty_clob()"); //oracle-specific
    s.commit();
    
    s.begin();
    rec = s.find(TEST_CLOB3.meta, SQueryMode.SFOR_UPDATE, 6);
    SormUtils.writeToClob(s, rec, "SOMECLOB", str256k);
    s.commit();
    
    s.begin();
    rec = s.find(TEST_CLOB3.meta, SQueryMode.SFOR_UPDATE, 6);
    String str = SormUtils.readFromClob(s, rec, "SOMECLOB");
    s.commit();
    
    s.close();        
  }
  
  public static class TEST_CLOB3
  extends SRecordInstance
  {
    static SRecordMeta<TEST_CLOB3> meta = 
      new SRecordMeta<TEST_CLOB3>(TEST_CLOB3.class, "TEST_CLOB3");
      
    static SFieldInteger ID = new SFieldInteger(meta, "ID", SFieldFlags.SPRIMARY_KEY);
    static SFieldString NAME = new SFieldString(meta, "NAME", 32);
    static SFieldClob2 SOMECLOB = new SFieldClob2(meta, "SOMECLOB", SFieldFlags.SUNQUERIED);
    
    @Override
    public SRecordMeta<?> getMeta() {
      return meta;
    }
    
  }
  
}
