package org.hmn.jrsd.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.StringReader;
import java.util.Properties;

import javax.sql.DataSource;

import oracle.jdbc.pool.OracleDataSource;

import org.apache.log4j.Appender;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.TTCCLayout;

import simpleorm.sessionjdbc.SSessionJdbc;
import junit.framework.TestCase;

/*
 * 
 * Cree par Harry Karadimas a 20 sept. 2006
 */
/**
 * 
 * @author karadimas
 */
public class OraJrsdTestCase
extends TestCase
{
  static Properties cxnProps = null;
  static DataSource ds = null;
  
  static final public void connect() throws Exception
  {
    if (cxnProps == null) {
      cxnProps = new Properties();
      cxnProps.load(new FileInputStream("local/ora-cxn.properties"));
      String url = cxnProps.getProperty("url");
      TestUtils.loadDriver(cxnProps);
      OracleDataSource ods = new OracleDataSource();
      ods.setURL(url);
      ods.setUser(cxnProps.getProperty("user"));
      ods.setPassword(cxnProps.getProperty("password")); 
      ds = ods;
    }    
  }
  
  /**
   * @see junit.framework.TestCase#setUp()
   */
  protected void setUp()
  throws Exception 
  {
    //add an "A2T" file appender to record output of the tests
    Appender apndr = Logger.getRootLogger().getAppender("A2T");
    if (apndr == null) {
      String fn = "test"+File.separator+"internaltests.log";
      //open in overwrite mode, as is is just for tests.
      FileAppender newAppender = 
        new FileAppender(new TTCCLayout(), fn, false);            
      newAppender.setThreshold(Level.DEBUG);
      Logger.getRootLogger().addAppender(newAppender);
      //SLogLog4jBridge.useSLogLog4jBridge();
    }
    connect();
  }
  
  @Override
  protected void tearDown() 
  throws Exception 
  {
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    if (session != null) session.close();
  }
  
  SSessionJdbc beginConnection(String name)
  {
    SSessionJdbc session = SSessionJdbc.open(ds, name);    
    session.begin();    
    return session;
  }
  
  void endConnection()
  {
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    session.commit();
    session.detachFromThread();
    session.close();
  }
  
  boolean tableExists(String name)
  throws Exception
  { return org.hmn.jrsd.rt.SormUtils.tableExists(name); }
  
  void ddl(String sql)
  throws Exception
  { org.hmn.jrsd.rt.SormUtils.ddl(sql); }
  
  /**
   * Soumettre plusieurs declarations, separees par des lignes ne
   * contenant que ";"
   * @param sql
   */
  void ddls(String sql)
  {
    StringBuffer sb = new StringBuffer();
    BufferedReader in = new BufferedReader(new StringReader(sql));
    String line;
    try {
      while ((line = in.readLine()) != null) {
        if (line.trim().equals(";")) {
          ddl(sb.toString());
          sb = new StringBuffer();
        }
        else {
          sb.append(line);
        }
      }
      String lastSql = sb.toString();
      if (lastSql.trim().length() > 0) ddl(lastSql);
    }
    catch (Exception ex) {
      throw new Error("Unexpected", ex);
    }
  }
  
}
