package org.hmn.jrsd;

import java.util.HashMap;
import java.util.zip.CRC32;

import org.hmn.jrsd.rt.JrsdRtUtils;

/*
 * 
 * Cree par Harry Karadimas a 30 juil. 2006
 */
/**
 * Column type definition. The name of the definition is used to select
 * the appropriate template when generating. Note that the type here is not
 * really SQL, but rather is used to generate the most appropriate SQL type,
 * and also the most appropriate Simpleorm accessor methods.
 * @author karadimas
 */
public class ColumnTypeDef
extends SqlElementDef
{
  private static HashMap<String, String> equivTypes = new HashMap<String, String>();
  
  static {
    equivTypes.put("integer", "INTEGER");
    equivTypes.put("int", "INTEGER");
    equivTypes.put("date", "DATE");
    equivTypes.put("number", "NUMBER");
    equivTypes.put("char", "CHAR");
    equivTypes.put("varchar", "VARCHAR");
    equivTypes.put("primitive_double", "DOUBLE");
    equivTypes.put("double", "DOUBLE");
    equivTypes.put("bytes", "BYTES");
    equivTypes.put("blob", "BLOB");
    equivTypes.put("clob", "CLOB");
    equivTypes.put("primitive_long", "LONG");
    equivTypes.put("long", "LONG");
    equivTypes.put("primitive_booleanchar", "CHAR");
    equivTypes.put("booleanchar", "CHAR");
    equivTypes.put("primitive_booleanbit", "BIT");
    equivTypes.put("booleanbit", "BIT");
    equivTypes.put("timestamp", "TIMESTAMP");
    equivTypes.put("time", "TIME");
    equivTypes.put("other", "OTHER");
  }
  
  String typeName;
  String param1;
  String scale;
  String la,lb,lc,ld,le,lf;
  
  /**
   * 
   */
  public ColumnTypeDef(
      String typeName, String precision, String scale)
  {
    this.typeName = typeName;
    this.param1 = precision;
    this.scale = scale;    
  }
    
  /**
   * A synonym for param1. 
   * @return getParam1()
   */
  public String getPrecision()
  {
    return param1;
  }
  public void setPrecision(String precision)
  {
    this.param1 = precision;
  }
  public String getParam1() { return param1; }
  
  public String getScale()
  {
    return scale;
  }
  public void setScale(String scale)
  {
    this.scale = scale;
  }
  public String getTypeName()
  {
    return typeName;
  }
  public void setTypeName(String typeName)
  {
    this.typeName = typeName;
  }
  public String getLa()
  {
    return la;
  }
  public void setLa(String la)
  {
    this.la = la;
  }
  public String getLb()
  {
    return lb;
  }
  public void setLb(String lb)
  {
    this.lb = lb;
  }
  public String getLc()
  {
    return lc;
  }
  public void setLc(String lc)
  {
    this.lc = lc;
  }
  public String getLd()
  {
    return ld;
  }
  public void setLd(String ld)
  {
    this.ld = ld;
  }
  public String getLe()
  {
    return le;
  }
  public void setLe(String le)
  {
    this.le = le;
  }
  public String getLf()
  {
    return lf;
  }
  public void setLf(String lf)
  {
    this.lf = lf;
  }
  
  public void update(CRC32 crc)
  {
    JrsdRtUtils.update(crc, param1);
    JrsdRtUtils.update(crc, scale);
    JrsdRtUtils.update(crc, typeName);
    JrsdRtUtils.update(crc, la);
    JrsdRtUtils.update(crc, lb);
    JrsdRtUtils.update(crc, lc);
    JrsdRtUtils.update(crc, ld);
    JrsdRtUtils.update(crc, le);
    JrsdRtUtils.update(crc, lf);
  }
  
  /*
   * Extract from the simpleorm template file (jrsdsql.st) :
      integer(t) ::= "integer$t.la$"
      int(t) ::= "integer$t.la$"
      date(t) ::= "date$t.la$"
      number(t) ::= <<
        number($t.precision$$if(t.scale)$, $t.scale$$endif$)
      >>
      char(t) ::= "char($t.precision$)"
      varchar(t) ::= "varchar($t.precision$)"
      primitive_double(t) ::= "double"
      double(t) ::= "double"
      bytes(t) ::= "raw$if(t.precision)$($t.precision$)$endif$"
      blob(t) ::= "blob"
      clob(t) ::= "clob"
      primitive_long(t) ::= "long"
      long(t) ::= "long"
      primitive_booleanchar(t) ::= "char(1)"
      booleanchar(t) ::= "char(1)"
      primitive_booleanbit(t) ::= "bit"
      booleanbit(t) ::= "bit"
      timestamp(t) ::= "timestamp"
      time(t) ::= "time"
      other(t) ::= " /* $t.param1$ * /"
   */
  public String inferEquivalentSqlTypeName()
  {
    String equiv = equivTypes.get(String.valueOf(typeName).toLowerCase());
    if (equiv != null) return equiv;
    return "?";
  }

  /**
   * Try to get what would be the closest to the SQL DDL.
   * @return
   */
  public String inferEquivalentSqlType()
  {
    StringBuffer sb = new StringBuffer();
    sb.append(inferEquivalentSqlTypeName());
    if (typeName.equalsIgnoreCase("booleanchar")) {
      sb.append("(1)"); //special case 
    }
    else {
      if (!isEmpty(param1) 
          && !typeName.equalsIgnoreCase("blob")
          && !typeName.equalsIgnoreCase("clob")) {
        sb.append('(');
        sb.append(param1);
        if (!isEmpty(scale)) {
          sb.append(',');
          sb.append(scale);
        }
        sb.append(')');
      }      
    }
    return sb.toString();
  }
  
  /** trivial method to test if a string is null or contains only spaces */
  public static final boolean isEmpty(String str) {
    return str == null || str.trim().equals("");
  }
  
}
