package org.hmn.jrsd;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/*
 * 
 * Cree par Harry Karadimas a 27 juil. 2006
 */
/**
 * The complete jrsd spec, produced by the {@link JrsdParser}.
 * @author karadimas
 */
public class JrsdSpec
{  
  String specFile;
  Date specDate;
  List<TableDef> tableDefs;
  ArrayList<JrsdError> errors = new ArrayList<JrsdError>();
  HashMap<String,MappingDecl> colListsByAlias =
    new HashMap<String, MappingDecl>();
  ArrayList<String> imports = new ArrayList<String>();
  boolean booleanCharAltUsed = false;
  String databaseType;
  String databaseSubtype;
  String fieldPrefix = "";
  
  public JrsdSpec()
  {
    this.tableDefs = new ArrayList<TableDef>();
  }
  
  public JrsdSpec(List<TableDef> tableDefs)
  {
    this.tableDefs = tableDefs;
  }
  
  public void addImport(String imp) { imports.add(imp); }
  
  public List<String> getImports() { return imports; }
  
  public void addGlobalColumnList(String alias, String name)
  {
    MappingDecl gfl = new MappingDecl("global", name, alias);
    gfl.setJrsdSpec(this);
    colListsByAlias.put(alias, gfl);
  }
  
  public MappingDecl getGlobalColumnList(String alias)
  {
    return colListsByAlias.get(alias);
  }
  
  public List<MappingDecl> getGlobalColumnLists()
  {
    return new ArrayList<MappingDecl>(colListsByAlias.values());
  }
  
  public List<String> getGlobalColumnListAliases()
  {
    return new ArrayList<String>(colListsByAlias.keySet());
  }
  
  public void addTableDef(TableDef td)
  {
    tableDefs.add(td);
    td.setJrsdSpec(this);
  }
    
  public int getErrorCount() { return errors.size(); }
  
  public List<TableDef> getTableDefs() { return tableDefs; }
  
  public List<MappingDecl> getMappingDeclList()
  {
    ArrayList<MappingDecl> mdcls = new ArrayList<MappingDecl>();
    for (Iterator<TableDef> iter = tableDefs.iterator(); iter.hasNext();) {
      TableDef td = iter.next();
      if (td.getMappingDecls() != null) {
        mdcls.addAll(td.getMappingDecls());
      }
    }
    return mdcls;
  }
  
  public MappingDecl[] getMappingDecls()
  {
    List<TableDef> tds = getTableDefs();
    MappingDecl[] da = tds.toArray(new MappingDecl[tds.size()]);
    return da;
  }
  
  public void addError(JrsdError e)
  {
    errors.add(e);
  }

  /**
   * Get the date of this specification file.
   * @return
   */
  public Date getSpecDate() {
    return specDate;
  }
  
  public void setSpecDate(Date specDate) {
    this.specDate = specDate;
  }

  public String getDatabaseType() {
    return databaseType;
  }

  public void setDatabaseType(String databaseType) {
    this.databaseType = databaseType;
  }

  public String getDatabaseSubtype() {
    return databaseSubtype;
  }

  public void setDatabaseSubtype(String databaseSubtype) {
    this.databaseSubtype = databaseSubtype;
  }

  public String getSpecFile() {
    return specFile;
  }

  public void setSpecFile(String specFile) {
    this.specFile = specFile;
  }

  public String getFieldPrefix() {
    return fieldPrefix;
  }

  public void setFieldPrefix(String fieldPrefix) {
    this.fieldPrefix = fieldPrefix;
  }

}
