package org.hmn.jrsd.rt;

import java.util.ArrayList;
import java.util.List;

/**
 * A node representing an SQL binary operator, like "X > ?" or "Z = 3"
 * @author karadimas
 */
public class BinOpExpr
extends SqlExpr
{
  /**
   * Is an empty string the same as NULL ? Default : true. This is especially
   * tricky and is the case with oracle. If you SELECT WHERE X=?, and
   * you use setString(1, ""), bad luck ! This will not work... You must
   * output SELECT WHERE X IS NULL. BinOpExpr handles this case when
   * this flag is true. Beware that this value is static so it holds for ALL
   * your binary operators. 
   */
  public static boolean emptyStringIsNull = true;
  
  String colName;
  String op;
  Object param;
  
  public BinOpExpr(String colName, String op, Object param)
  {
    this.colName = colName;
    this.op = op;
    //replace "" with null if "" is null (this is the case with Oracle)
    if ("".equals(param) && emptyStringIsNull) {
      param = null;
    }
    this.param = param;
  }
  
  /**
   * @see org.hmn.jrsd.rt.SqlExpr#getText()
   */
  public String getText() 
  {
    StringBuffer sb = new StringBuffer();
    sb.append(colName);
    sb.append(' ');
    if (op.equals("=") & param == null) {
      sb.append("IS NULL");
    }
    else {
      sb.append(op);
      sb.append(" ?");
    }
    return sb.toString();
  }

  /**
   * Add all parameters, except for parameters where operator is "=" and 
   * value NULL, because in this case "IS NULL" is output instead of "=?".
   * @see org.hmn.jrsd.rt.SqlExpr#getParameters()
   */
  List<Object> getParameters()
  {
    ArrayList<Object> r = new ArrayList<Object>();
    if (!(op.equals("=") & param == null)) {
      r.add(param);
    }
    return r;
  }
  
}
