package org.hmn.jrsd.rt;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/*
 * 
 * Cree par Harry Karadimas a 5 oct. 2006
 */
/**
 * A list of expressions, separated by a separator. Used by and, or, expressions
 * but could also be used to chain multiple expressions. If the list is
 * not empty, the list is enclosed in parentheses.
 * @author karadimas
 */
public class ListExpr
extends SqlExpr
{
  String sep = "";
  ArrayList<SqlExpr> expressions = new ArrayList<SqlExpr>();

  public ListExpr()
  {    
  }
  
  public ListExpr(String separator)
  {
    this.sep = separator;
  }
  
  public ListExpr add(SqlExpr e) { expressions.add(e); return this; } 
  
  public ListExpr add(String txt)
  {
    add(new LiteralExpr(txt));
    return this;
  }
  
  /**
   * Convenience method, to add a string as a text constant (the method calls
   * add("'" + txt + "'"). This is useful to build a list. Ex :
   * ListExpr lexp = new ListExpr(",");
   * lexp.addStringLit("A");
   * lexp.addStringLit("B");
   * lexp.addStringLit("C");
   * String str = lexp.getText(); // str is "('A' , 'B' , 'C')"
   * @param txt
   * @return
   */
  public ListExpr addStringLit(String txt)
  {
    return add("'" + txt + "'");
  }
  
  /**
   * Get the content of this list expression, enclosed in parentheses.
   * @see org.hmn.jrsd.rt.SqlExpr#getText()
   */
  public String getText() 
  {
    StringBuffer sb = new StringBuffer();
    for (Iterator<SqlExpr> iter = expressions.iterator(); iter.hasNext();) {
      SqlExpr e = iter.next();
      if (sb.length() > 0) sb.append(" "+sep+" ");
      else sb.append('(');
      sb.append(e.getText());
    }
    if (sb.length() > 0) sb.append(')');        
    return sb.toString();
  }

  /**
   * @see org.hmn.jrsd.rt.SqlExpr#getParameters()
   */
  public List<?> getParameters() 
  {
    ArrayList<Object> r = new ArrayList<Object>();
    for (Iterator<?> iter = expressions.iterator(); iter.hasNext();) {
      SqlExpr e = (SqlExpr) iter.next();
      r.addAll(e.getParameters());
    }
    return r;
  }
  
  public boolean isEmpty() { return expressions.isEmpty(); }
  
}
