package org.hmn.jrsd.rt;

/**
 * A qualified column (a column that knowns its table name and uses it).
 * Kept in the runtime because the "quoteIfNecessary" method might be
 * useful for any (semi-)hand-crafted sql statements. 
 * @author karadimas
 */
public class QColumn
{
  private String columnName;
  private String tableName;
  
  public QColumn(String tableName, String columnName)
  {
    this.tableName = tableName;
    this.columnName = columnName;
  }
  public String getColumnName()
  {
    return columnName;
  }
  public void setColumnName(String columnName)
  {
    this.columnName = columnName;
  }
  public String getTableName()
  {
    return tableName;
  }
  public void setTableName(String tableName)
  {
    this.tableName = tableName;
  } 
  
  /** 
   * quote if necessary (if n does not begin with a letter, and/or 
   * contains something else than letters, numbers, '_'. Useful to quote
   * names in SQL statements only if necessary. This makes the SQL query more
   * readable. 
   */
  static final public String quoteIfNecessary(String n)
  {
    int ec = 0;
    for (int i = 0; i < n.length(); i++) {
      char c = n.charAt(i);
      if ( !((c >= 'a' && c <= 'z') || 
             (c >= 'A' && c <= 'Z') ||
             (i > 0 && c >= '0' && c <= '9') ||
             (i > 0 && c == '_') ) )  {
        ec++; break;
      }
    }//for
    if (ec == 0) return n;
    return "\""+n+"\"";
  }
  
  /**
   * Make a name suitable for a java class, by transforming all chars that
   * are not alphanumeric into an underscore.
   * @param n
   * @return
   */
  static final public String makeNameForJava(String n)
  {
    StringBuffer sb = new StringBuffer();    
    for (int i = 0; i < n.length(); i++) {
      char c = n.charAt(i);
      if ( ((c >= 'a' && c <= 'z') || 
            (c >= 'A' && c <= 'Z') ||
            (i > 0 && c >= '0' && c <= '9') ||
            (i > 0 && c == '_') ) )  {
        sb.append(c);
      }
      else {
        sb.append('_');
      }
    }//for
    return sb.toString();
  }
  
  /**
   * Return qualified representation (table.column), the names will be enclosed
   * in quotes if they don't start with a letter and/or if they don't contain
   * only letters, numbers and underscores.
   * @see java.lang.Object#toString()
   */
  public String toString()
  {
    return quoteIfNecessary(tableName)+"."+quoteIfNecessary(columnName);

  }
}
