/**
 * 
 */
package org.hmn.jrsd.rt;

import simpleorm.drivers.SDriverOracle;

/**
 * A driver for Oracle that declares it does not support locking. This avoids
 * "FOR UPDATE" clauses, which is not that bad because locking is built-in in 
 * Oracle, and locking rows for a long period is troublesome. FOR UPDATE
 * is recommended by Oracle to alter a LOB (CLOB or BLOB), but for other uses,
 * the normal transactional behaviour should be sufficient. I have had 
 * frequent deadlocks with SimpleOrm default locking behaviour under Oracle
 * (Althouth things are much better now with recent versions).
 * Example usage :
 * <pre>
 *    OracleConnectionPoolDataSource ocpds = new OracleConnectionPoolDataSource();
 *    ocpds.setURL(url);
 *    ocpds.setUser(jdbcUser);
 *    ocpds.setPassword(password);
 *    OracleConnectionCacheImpl occi = new OracleConnectionCacheImpl(ocpds);
 *    occi.setMaxLimit(maxJdbcConnections);    
 *    SDataSource ds = new SDataSourceJavaX(occi);
 *    //force the use of the no locking driver.
 *    ds.setSDriverName(SDriverOracleNoLock.NAME);
 * </pre>
 * @author karadimas
 */
public class SDriverOracleNoLock
extends SDriverOracle
{
  //note that referencing this static variable is sufficient to instanciate
  //the SDriverOrackeNoLock class and trigger the static initializer, who
  //registers the driver !
  public static final String NAME = "Oracle SimpleORM driver without locking";

  static {
    new SDriverOracleNoLock().registerDriver();
  }
  
  static public void init() {
    //does nothing, but ensures static {} block initializer will have been
    //called.
  }
  
  protected String driverName()
  {
    return NAME;
  }
  
  /** force optimistic locking */
  public boolean supportsLocking() { return false; }
  
}
