package org.hmn.jrsd.rt;

import simpleorm.dataset.SFieldFlags;
import simpleorm.dataset.SFieldString;
import simpleorm.dataset.SRecordMeta;

/*
 * 
 * Cree par Harry Karadimas a 7 oct. 08
 */
/**
 * An SField that uses finite strings. It truncates them to a maximum size.
 */
public class SFieldFiniteString
extends SFieldString
{

  private static final long serialVersionUID = 259962839464853085L;

  public SFieldFiniteString(SRecordMeta<?> meta, String columnName,
      int maxSize, SFieldFlags... pvals) {
    super(meta, columnName, maxSize, pvals);
  }

  /**
   * Limit the string to maxLen chars.
   * @param str the string to limit (null is allowed)
   * @param maxLen the maximum allowed length for the string
   * @return the limited string
   */
  public static String limit(String str, int maxLen)
  {
    if (str == null) return null;
    if (str.length() > maxLen) return str.substring(0, maxLen);
    return str;
  }
  
  /**
   * Limit string size before writing to database
   * @see simpleorm.dataset.SFieldMeta#writeFieldValue(java.lang.Object)
   */
  @Override
  public Object writeFieldValue(Object value) {
    if (value == null) return null;
    return limit(value.toString(), getMaxSize());
  }
  
  /**
   * Convert this to a FiniteString.
   * @see simpleorm.dataset.SFieldString#convertToDataSetFieldType(java.lang.Object)
   */
  @Override
  protected Object convertToDataSetFieldType(Object raw) {
    String rawStr = raw == null ? null : raw.toString();
    FiniteString fs = new FiniteString(rawStr, getMaxSize());
    return fs;
  }
  
}
