package org.hmn.jrsd.rt;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

/*
 * 
 * Cree par Harry Karadimas a 5 oct. 2006
 */
/**
 * Ancestor of sql expressions. This is a simple representation for
 * expressions, the main use of this representation is to ease the 
 * representation of SQL expressions where NULLS are involved :
 * when there is "=" and the parameter is not null, "=?" is emitted
 * in the SQL, and setObject(... , value) is issued on the prepared
 * statement, but when the parameter is null, "IS NULL" is emitted
 * in the SQL, and no setObject call takes place. Sometimes this is
 * better achieved with Simpleorm expressions, however you can use
 * these sql expressions to build raw clauses that won't fit in
 * Simpleorm queries (like queries for Oracle LOBs for example, or to
 * implement OR clauses which must be built by hand with simpleorm).
 * @author karadimas
 */
abstract public class SqlExpr
{
  /** get the final expression */
  abstract public String getText();
  
  /** get all the parameters, except those that are null for "=" */
  abstract List<?> getParameters();
  
  /** set the collected parameters */
  public void setParameters(int startIndex, PreparedStatement ps)
  throws SQLException
  {
    int i = startIndex;
    List<?> params = getParameters();
    for (Object obj : params) ps.setObject(i++, obj);    
  }
  
}
