package org.hmn.jrsd.rt;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Iterator;
import java.util.List;

/*
 * 
 * Cree par Harry Karadimas a 5 oct. 2006
 */
/**
 * A class to help building WHERE clauses, in another manner than simpleorm.
 * Least-surprise, trivial implementation. The main goal is to emis IS NULL
 * when the value is null, and = ? otherwise. This disturbs the order in which
 * parameters are emitted, so the parameter list must be set accordingly. This
 * Where help does just that. 
 * @author karadimas
 */
public class WhereHelper
{
  SqlExpr expression;
  
  public WhereHelper() {}
  
  public WhereHelper(SqlExpr e) { expression = e; }
  
  /**
   * Get the SQL text for the expression
   * @return
   */
  public String getText()
  {
    if (expression == null) return null;
    return expression.getText();
  }
  
  /**
   * Get the parameters for the expression, as List
   * @return
   */
  public List<?> getParameters()
  {
    if (expression == null) return null;
    return expression.getParameters();
  }
  
  /**
   * Get the parameters for the expression, as an Array
   * @return
   */
  public Object[] getParameterArray()
  {
    if (expression == null) return null;
    return expression.getParameters().toArray();    
  }
  
  /**
   * Get the sql expression used in this where clause.
   */
  public SqlExpr getSqlExpr() {
    return expression;
  }
  public void setSqlExpr(SqlExpr sqlExpr) {
    this.expression = sqlExpr;
  }
    
  /**
   * Set parameters. If the parameter is not fully qualified and is null,
   * calls setNull with an SQL type of VARCHAR (check that this is OK for
   * your database, this type parameter is usually ignored by JDBC for what I 
   * know). Also check that setObject() is OK for all your values in a given
   * database.
   * @param ps
   * @throws SQLException
   */
  public void setParameters(PreparedStatement ps)
  throws SQLException
  {
    int i = 1;
    if (expression == null) return;
    List<?> params = expression.getParameters();
    for (Iterator<?> iter = params.iterator(); iter.hasNext();) {
      Object p = (Object) iter.next();
      if (p instanceof FqParameter) {
        FqParameter fqp = (FqParameter) p;
        if (fqp.getValue() == null) ps.setNull(i, fqp.getType());
        else ps.setObject(i, fqp.getValue());
      }
      else {
        if (p == null) ps.setNull(i, Types.VARCHAR);
        else ps.setObject(i, p);
      }
      i++;
    }//for    
  }
  
}
