package org.hmn.jrsd.rt.dbops;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.hmn.jrsd.rt.ReflectionHelpers;

import simpleorm.sessionjdbc.SSessionJdbc;
import simpleorm.utils.SException;
import simpleorm.dataset.SQuery;
import simpleorm.dataset.SRecordInstance;

/*
 * 
 * Cree par Harry Karadimas a 2 mai 08
 */
/**
 * Read records using an {@link SQuery}, and if there was a class declared for
 * transport objects, write them to a collection of transport objects. If no 
 * class was declared, store the simpleorm records in the collection.
 */
public class DbRecordQueryOperation
extends DbListOperation
{
  SQuery<SRecordInstance> q;
  Class<?> transportClass;
  
  public DbRecordQueryOperation(
      SQuery<SRecordInstance> q, Class<?> transportClass)
  {
    this.q = q;
    this.transportClass = transportClass;
  }
  
  public DbRecordQueryOperation(SQuery<SRecordInstance> q)
  {
    this(q, null);
  }
  
  public List<?> execute()
  throws SQLException
  {
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    
    List<SRecordInstance> rs = session.query(q);
    ArrayList<Object> r = new ArrayList<Object>();
    for (SRecordInstance rec : rs) {  
      if (transportClass == null) r.add(rec);
      else {
        Object tobj = makeTransportObject();
        try {
          ReflectionHelpers.callVoid("writeTo", rec, tobj);
        }
        catch (Exception e) {
          throw new SException.Data(e);
        }
        r.add(tobj);        
      }
    }//for
    return r;
  }

  private final Object makeTransportObject()
  {
    try {
      return transportClass.newInstance();
    }
    catch (Exception e) {
      throw new SException.Data(e);
    }
  }
}
