package org.hmn.jrsd.rt.dbops;

import java.sql.SQLException;

import org.hmn.jrsd.rt.PrimaryKeysRetrievable;
import org.hmn.jrsd.rt.ReflectionHelpers;

import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;
import simpleorm.sessionjdbc.SSessionJdbc;
import simpleorm.utils.SException;

/*
 * 
 * Cree par Harry Karadimas a 2 mai 08
 */
/**
 * Standard record update operation. Returns 1 if the record was updated
 * correctly. If the record does not exist, throws an {@link SQLException}.
 * If exceptionRaisedIfNotExists is set to false, and the record does not exist,
 * returns 0.
 * The SRecordInstance associated with the given "meta" must have a "readFrom"
 * method for the given transport object.
 */
public class DbRecordUpdateOperation
extends DbUpdateOperation
{
  SRecordMeta<?> meta;
  Object keys;
  Object transportObject;
  long sqy_bitSet;
  boolean exceptionRaisedIfNotExists = true;
  
  public DbRecordUpdateOperation(
      SRecordMeta<?> meta, Object keys, Object transportObject)
  {
    this(meta, keys, transportObject, 0L);
  }
  
  /**
   * A record update operation on a transport object that knows how to retrieve
   * its primary key values.
   * @param meta
   * @param transportObject
   */
  public DbRecordUpdateOperation(
      SRecordMeta<?> meta, PrimaryKeysRetrievable transportObject)
  {
    this(meta, transportObject.getPrimaryKeyValues(), transportObject, 0L);
  }
  
  public DbRecordUpdateOperation(
      SRecordMeta<?> meta, Object keys, Object transportObject, long sqy_bitSet)
  {
    this.meta = meta;    
    this.keys = keys;
    this.sqy_bitSet = sqy_bitSet;
  }
  
  public int execute()
  throws SQLException
  {
    SSessionJdbc session = SSessionJdbc.getThreadLocalSession();
    SRecordInstance rec = session.findOrCreate(meta, keys);
    if (rec.isNewRow()) {
      if (exceptionRaisedIfNotExists) {
        throw new SQLException(
            "Record does not exist "+meta.getTableName()+"#"+keys);
      }
      else return 0;
    }
    try {
      ReflectionHelpers.callVoid("readFrom", rec, transportObject);
      return 1;
    }
    catch (Exception e) {
      throw new SException.Data(e);
    }    
  }

  public boolean isExceptionRaisedIfNotExists() {
    return exceptionRaisedIfNotExists;
  }

  public void setExceptionRaisedIfNotExists(boolean exceptionRaisedIfExists) {
    this.exceptionRaisedIfNotExists = exceptionRaisedIfExists;
  }

}
