/**
 * 
 */
package org.hmn.jrsd.rt.rows;

import java.io.Serializable;
import java.util.HashMap;

import org.hmn.jrsd.rt.JrsdRtUtils;

/**
 * A result from a jdbc result array. You can get the values in three possible
 * ways :
 * <ol>
 *   <li>getRawValue : the original value, untouched.
 *   <li>getValueAsString : the value as String, but with HTML special chars escaped
 *   <li>getFormattedValue : the cell value, formatted with the given formatter
 * </ol>
 * The cell also contains a HashMap that can be used to store further arbitrary
 * attributes (needed for formatting, for example).
 * @author KARADIMAS
 */
public class JdbcResultCell
implements Serializable
{
  /**
   * 
   */
  private static final long serialVersionUID = -27976742247104984L;
  
  JdbcResultColumn column;
  int rowIndex;
  Object valueObject;
  JdbcResultRow cells;
  HashMap<?,?> st = null;
  
  public JdbcResultCell() {}
  
  public String getEncoding() {
    return cells.rows.results.getEncoding();
  }
  
  private final JdbcResults getJdbcResults() { return cells.rows.results; }
  
  /**
   * 
   */
  public JdbcResultCell(JdbcResultColumn col, int rowIx, Object rawVal) {
    this.column = col;
    this.rowIndex = rowIx;
    this.valueObject = rawVal;
  }
  
  public JdbcResultColumn getColumn() {
    return column;
  }
  public void setColumn(JdbcResultColumn column) {
    this.column = column;
  }
  public Object getRawValue() {
    return valueObject;
  }
  public void setRawValue(Object rawValue) {
    this.valueObject = rawValue;
  }
  public int getRowIndex() {
    return rowIndex;
  }
  public void setRowIndex(int rowIndex) {
    this.rowIndex = rowIndex;
  }
  public String getValueAsString()
  {
    return JrsdRtUtils.encodeWithEntities(String.valueOf(valueObject));
  }
  public String getFormattedValue()
  {
    String label = column.getLabel();
    CellFormat fmt = getJdbcResults().getFormat(label);
    if (fmt == null) return getValueAsString();
    return fmt.format(this);
  }
  
  public JdbcResultRow getRow() {
    return cells;
  }
  
  public void setRow(JdbcResultRow cells) {
    this.cells = cells;
  }
  
  public String toString() {    
    return getFormattedValue();
  }

  /**
   * Additional values for pre-formatted values you can retrieve (useful
   * for the stringtemplate library).
   * @return
   */
  public HashMap<?,?> getSt() {
    return st;
  }

  public void setSt(HashMap<?,?> st) {
    this.st = st;
  }
  
}
