package org.hmn.jrsd.rt.rows;

import java.util.ArrayList;
import java.util.List;

/*
 * 
 * Cree par Harry Karadimas a 5 fvr. 2007
 */
/**
 * All result rows. Is not Serializable because must be reconstructed on the
 * client side, using a {@link JdbcResults} object.
 * @author karadimas
 */
public class JdbcResultRows
{
  JdbcResults results;
  /** shared with JdbcResultRow */
  ArrayList<JdbcResultCell> recycledCells = new ArrayList<JdbcResultCell>();
  ArrayList<JdbcResultRow> rowList = new ArrayList<JdbcResultRow>();

  public JdbcResultRows(JdbcResults results) 
  {
    this(results, JdbcResultCell.class);
  }
  
  /**
   * Construct the rows , and use the given class to
   * construct the cells that will get recycled. 
   */
  public JdbcResultRows(
      JdbcResults results, Class<? extends JdbcResultCell> jdbcResultCellClass) 
  {
    this.results = results;
    //initialize
    int rc = results.getRowCount();
    for (int i = 0; i < rc; i++) {      
      JdbcResultRow cells = null;
      cells = new JdbcResultRow(this, i);
      rowList.add(cells);
    }
    int cc = results.getColumns().getColumnCount();
    while (recycledCells.size() < cc) {
      JdbcResultCell cell = null;
      try { cell = jdbcResultCellClass.newInstance(); }
      catch (InstantiationException e) { throw new RuntimeException(e); }
      catch (IllegalAccessException e) { throw new RuntimeException(e); }
      recycledCells.add(cell);    
    }
  }
  
  public JdbcResults getResults() {
    return results;
  }
  
  public JdbcResultColumns getColumns() {
    return results.getColumns();
  }
  
  /**
   * @return A list of the result rows
   */
  public List<JdbcResultRow> getList()
  {
    return rowList;    
  }
    
}
