package org.hmn.jrsd.test;

import org.hmn.jrsd.rt.FiniteString;

import simpleorm.dataset.SFieldFlags;
import simpleorm.dataset.SFieldInteger;
import simpleorm.dataset.SFieldString;
import simpleorm.dataset.SRecordInstance;
import simpleorm.dataset.SRecordMeta;

import junit.framework.Assert;
import junit.framework.TestCase;

public class FiniteStringTests 
extends TestCase
{
  String s1 = "Hello, world !";

  public void testHashCode() {
    FiniteString fs1 = new FiniteString(s1, 64);
    FiniteString fs2 = new FiniteString(s1, 8);
    Assert.assertEquals(s1.hashCode(), fs1.hashCode());
    Assert.assertFalse(s1.hashCode() == fs2.hashCode());    
  }

  public void testTruncate() {
    String ts1 = FiniteString.truncate(s1, 8);
    Assert.assertEquals("Hello, w", ts1);
  }

  public void testFiniteStringStringInt() {
    FiniteString fs1 = new FiniteString(s1, 8);
    Assert.assertEquals("Hello, w", fs1.toString());
  }

  public void testFiniteStringStringSFieldString() {
    FiniteString fs1 = new FiniteString(s1, SriForTests.myteststring);
    Assert.assertEquals("Hello, w", fs1.toString());
  }

  public void testFiniteStringSFieldString() {
    FiniteString fs1 = new FiniteString(s1, SriForTests.myteststring);
    Assert.assertEquals("Hello, w", fs1.toString());
  }

  public void testToString() {
    FiniteString fs1 = new FiniteString(s1, 8);
    Assert.assertEquals("Hello, w", fs1.toString());
    FiniteString fs2 = new FiniteString(null, 8);
    Assert.assertNull(fs2.toString());
  }

  public void testGetMaxLen() {
    FiniteString fs1 = new FiniteString(s1, 8);
    assertEquals(8, fs1.getMaxLen());
  }

  public void testMakeNew() {
    FiniteString fs1 = new FiniteString(s1, 8);
    FiniteString fs2 = fs1.makeNew("This is a string that will get truncated");
    assertEquals("This is ", fs2.toString());
  }

  public void testEqualsObject() {
    String str1 = "Hello World !";
    String str2 = "Hello ".concat("World !");
    assertFalse(str1 == str2);
    assertTrue(str1.equals(str2));
    FiniteString fs1 = new FiniteString(str1, 32);
    FiniteString fs2 = new FiniteString(str2, 64);
  //note maximum lengths are different, but fs1 and fs2 are considered equal
    assertTrue(fs1.equals(fs2)); 
  }

  /** static class just for tests */
  public static class SriForTests
  extends SRecordInstance
  {
    public static SRecordMeta<SriForTests> meta = 
      new SRecordMeta<SriForTests>(SriForTests.class, "srifortests");
    public static SFieldInteger srifortests_id = 
      new SFieldInteger(meta, "srifortests_id", SFieldFlags.SPRIMARY_KEY);
    public static SFieldString myteststring = 
      new SFieldString(meta, "myteststring", 8); 
    @Override
    public SRecordMeta<?> getMeta() { return meta; }
  }
  
}
